#! /usr/bin/python
#***********************************************************
#* Software License Agreement (BSD License)
#*
#*  Copyright (c) 2009, Willow Garage, Inc.
#*  All rights reserved.
#*
#*  Redistribution and use in source and binary forms, with or without
#*  modification, are permitted provided that the following conditions
#*  are met:
#*
#*   * Redistributions of source code must retain the above copyright
#*     notice, this list of conditions and the following disclaimer.
#*   * Redistributions in binary form must reproduce the above
#*     copyright notice, this list of conditions and the following
#*     disclaimer in the documentation and/or other materials provided
#*     with the distribution.
#*   * Neither the name of Willow Garage, Inc. nor the names of its
#*     contributors may be used to endorse or promote products derived
#*     from this software without specific prior written permission.
#*
#*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#*  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#*  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#*  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#*  POSSIBILITY OF SUCH DAMAGE.
#* 
#* Author: Eitan Marder-Eppstein
#***********************************************************
import roslib; roslib.load_manifest('application_client')
import rospy
import actionlib
import sys

from application_msgs.msg import RunApplicationAction, RunApplicationGoal

class ApplicationClient:
    def __init__(self):
        self.client = actionlib.SimpleActionClient('application_manager/run_application', RunApplicationAction)
        self.client.wait_for_server()

        #we want to make sure to register a callback on shutdown of the node
        rospy.on_shutdown(self.shutdown)

        self.application_name = rospy.get_param('~application_name')
        self.daemonize = rospy.get_param('~daemonize', False)

        self.intentional_shutdown = False

    def run_application(self):
        if self.application_name is None:
            rospy.logerr("You are attempting to run an application client without giving it the name \
                         of an application to launch. This won't work, this client won't do anything.")
            return

        goal = RunApplicationGoal()
        goal.daemonize = self.daemonize
        goal.application_name = self.application_name
        self.client.send_goal(goal, self.goal_done)

    def goal_done(self, terminal_state, result):
        if not self.intentional_shutdown:
            rospy.logwarn("The application associated with this client shut down without the client requesting it")
            sys.exit(0)

    def shutdown(self):
        rospy.loginfo("Shutting down application before the node is killed")
        self.intentional_shutdown = True
        self.client.cancel_goal()
        self.client.wait_for_result()
        rospy.loginfo("Sent shut down signal to application")
        sys.exit(0)

def main():
    rospy.init_node('application_client')
    app_client = ApplicationClient()
    app_client.run_application()
    rospy.spin()

if __name__ == '__main__':
    main()
