#!/usr/bin/python
# Software License Agreement (BSD License)
#
# Copyright (c) 2008, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of the Willow Garage nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

# Author: Wim Meeussen




from __future__ import with_statement

import roslib; roslib.load_manifest('application_manager')
import rospy
import actionlib
from actionlib_msgs.msg import GoalStatus
import sys
import optparse
from application_msgs.msg import *

def main():
   args = sys.argv
   args.remove(args[0])

   if len(args) == 0:
      print 'Usage: do.py list/start/stop/kill'
   else:
      if args[0] == 'list':
         ac = actionlib.SimpleActionClient('application_manager/list_applications', ListApplicationsAction)
         ac.wait_for_server(rospy.Duration(10.0))
         ac.send_goal_and_wait(ListApplicationsGoal())
         res = ac.get_result()
         if len(res.application_name) == 0:
            print "No applications available"
         else:
            print "The following appliations are managed by the application manager:"
            for name, status in zip(res.application_name, res.application_status):
               print "  - %s (%s)"%(name, status)

      if args[0] == 'start':
         for i in range(1, len(args)):
            ac = actionlib.SimpleActionClient('application_manager/run_application', RunApplicationAction)
            ac.wait_for_server(rospy.Duration(10.0))
            ac.send_goal(RunApplicationGoal(application_name = args[i]))
            while not ac.get_state() in [GoalStatus.ACTIVE, GoalStatus.REJECTED]:
               rospy.sleep(0.1)
            if ac.get_state() == GoalStatus.ACTIVE:
               print "Started application %s"%args[i]
            else:
               print "Failed to start application %s"%args[i]         
         

      if args[0] == 'stop':
         for i in range(1, len(args)):
            ac = actionlib.SimpleActionClient('application_manager/stop_application', StopApplicationAction)
            ac.wait_for_server(rospy.Duration(10.0))
            ac.send_goal_and_wait(StopApplicationGoal(application_name = args[i]))
            if ac.get_state() == GoalStatus.SUCCEEDED:
               print "Killed application %s"%args[i]
            else:
               print "Failed to kill application %s"%args[i]         

      if args[0] == 'kill':
         for i in range(1, len(args)):
            ac = actionlib.SimpleActionClient('application_manager/kill_application', KillApplicationAction)
            ac.wait_for_server(rospy.Duration(10.0))
            ac.send_goal_and_wait(KillApplicationGoal(application_name = args[i]))
            if ac.get_state() == GoalStatus.SUCCEEDED:
               print "Killed application %s"%args[i]
            else:
               print "Failed to kill application %s"%args[i]         




if __name__ == '__main__':
   rospy.init_node('application_manager_do')
   main()



