#! /usr/bin/python
# Software License Agreement (BSD License)
#
# Copyright (c) 2008, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of the Willow Garage nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

# Author: Wim Meeussen




from __future__ import with_statement

import roslib; roslib.load_manifest('application_manager')
import rospy
import threading
import actionlib
from actionlib_msgs.msg import GoalStatus
from application_msgs.msg import *



class ApplicationMonitor(threading.Thread):
    def __init__(self, app_cb):
        threading.Thread.__init__(self)
        self.lock = threading.RLock()
        self.app_cb = app_cb
        self.state = {}
        rospy.loginfo("Connecting to the application manager.")
        self.list_client = actionlib.SimpleActionClient(rospy.remap_name('application_manager') + '/list_applications', 
                                                       ListApplicationsAction)
        self.list_client.wait_for_server()
        rospy.loginfo("Connected to the application manager.")
        self.get_state()  # initialize state
        self.start()  # start running monitor


    def monitor_app(self, app_name):
        with self.lock:
            self.state[app_name] = None
            self.get_state()
    

    def get_state(self):
        # get current state
        self.list_client.send_goal_and_wait(ListApplicationsGoal(), rospy.Duration(20.0), rospy.Duration(2.0))
        res = self.list_client.get_result()
        last_state = {}
        for app, state in zip(res.application_name, res.application_status):
            last_state[app] = state

        # check if state changed
        with self.lock:
            for a in self.state:  # loop through all apps we're monitoring
                if a in last_state:  
                    if not self.state[a]:
                        self.state[a] = last_state[a]  # set state for first tiem
                    elif last_state[a] != self.state[a]:  # state changed
                        self.state[a] = last_state[a]
                        self.app_cb(a, self.state[a])  # callback
                    

    def run(self):
        rospy.loginfo('Running battery callbacks')
        while not rospy.is_shutdown():
            rospy.sleep(5.0)
            self.get_state()
