#! /usr/bin/python
#***********************************************************
#* Software License Agreement (BSD License)
#*
#*  Copyright (c) 2009, Willow Garage, Inc.
#*  All rights reserved.
#*
#*  Redistribution and use in source and binary forms, with or without
#*  modification, are permitted provided that the following conditions
#*  are met:
#*
#*   * Redistributions of source code must retain the above copyright
#*     notice, this list of conditions and the following disclaimer.
#*   * Redistributions in binary form must reproduce the above
#*     copyright notice, this list of conditions and the following
#*     disclaimer in the documentation and/or other materials provided
#*     with the distribution.
#*   * Neither the name of Willow Garage, Inc. nor the names of its
#*     contributors may be used to endorse or promote products derived
#*     from this software without specific prior written permission.
#*
#*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#*  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#*  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#*  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#*  POSSIBILITY OF SUCH DAMAGE.
#* 
#* Author: Eitan Marder-Eppstein
#***********************************************************
import roslib; roslib.load_manifest('ui_preempter')
import rospy
import actionlib

from application_msgs.msg import ApplicationAction
from std_msgs.msg import Empty

class UIPreempter:
    def __init__(self):
        self.server = actionlib.SimpleActionServer('action_ns', ApplicationAction, self.execute)
        self.preempt_pub = rospy.Publisher(rospy.remap_name('action_ns') + '/preempt_request', Empty, None, False, True)
        self.preempt_sub = rospy.Subscriber(rospy.remap_name('action_ns') + '/preempt_response', Empty, self.response_cb)
        self.response_heard = False

    def execute(self, goal):
        rospy.loginfo('Got an application goal')
        r = rospy.Rate(10.0)
        while not rospy.is_shutdown():
            if self.server.is_preempt_requested():
	        rospy.loginfo('Got a preempt request')
                self.preempt_ui_and_wait()
                self.server.set_preempted()
		return
            r.sleep()

    def preempt_ui_and_wait(self):
        #Make sure to reset response heard flag
        self.response_heard = False

        #Publish a preempt request
        rospy.loginfo('Sending a preempt request to the UI')
        self.preempt_pub.publish(Empty())

        #Wait for a response
        r = rospy.Rate(10.0)
        while not self.response_heard and not rospy.is_shutdown() and self.preempt_sub.get_num_connections() > 0:
            r.sleep()

        if self.preempt_sub.get_num_connections() == 0:
            rospy.loginfo("No one is subscribed to the UI preempter so it'll just shut down")
            return

        if rospy.is_shutdown():
            rospy.loginfo("Getting torn down, but I never got a response from the UI")
        else:
            rospy.loginfo("The ui for this application has given the OK for the application to be taken down")

    def response_cb(self, msg):
        self.response_heard = True

if __name__ == '__main__':
    rospy.init_node('ui_preempter')
    ui_p = UIPreempter()
    rospy.spin()
