#! /usr/bin/python
#***********************************************************
#* Software License Agreement (BSD License)
#*
#*  Copyright (c) 2009, Willow Garage, Inc.
#*  All rights reserved.
#*
#*  Redistribution and use in source and binary forms, with or without
#*  modification, are permitted provided that the following conditions
#*  are met:
#*
#*   * Redistributions of source code must retain the above copyright
#*     notice, this list of conditions and the following disclaimer.
#*   * Redistributions in binary form must reproduce the above
#*     copyright notice, this list of conditions and the following
#*     disclaimer in the documentation and/or other materials provided
#*     with the distribution.
#*   * Neither the name of Willow Garage, Inc. nor the names of its
#*     contributors may be used to endorse or promote products derived
#*     from this software without specific prior written permission.
#*
#*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#*  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#*  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#*  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#*  POSSIBILITY OF SUCH DAMAGE.
#* 
#* Author: Eitan Marder-Eppstein
#***********************************************************
import roslib; roslib.load_manifest('ui_responder')
import rospy

from std_msgs.msg import Empty

from PyQt4 import QtGui, QtCore

import sys

class UIResponder(QtGui.QMessageBox):
    preempted = QtCore.pyqtSignal()
    def __init__(self):
        QtGui.QMessageBox.__init__(self)
        self.timeout_seconds = rospy.get_param('~timeout', 60)
        self.name = rospy.get_param('~name', 'task')
        self.details = rospy.get_param('~details', 'Make sure it is safe to let the robot do something else.')

        self.pub = rospy.Publisher(rospy.remap_name('action_ns') + '/preempt_response', Empty)
        self.sub = rospy.Subscriber(rospy.remap_name('action_ns') + '/preempt_request', Empty, self.req_cb)
        self.preempted.connect(self.handle_preempt)
        self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
        self.end_now_button = self.addButton("Safe to end " + self.name, QtGui.QMessageBox.YesRole)
        self.defer_button = self.addButton("Remind me later", QtGui.QMessageBox.NoRole)
        self.setWindowTitle("Robot Task Preempt Request")
        self.setText("Another application or user needs to take control of the robot.")
        self.setInformativeText(self.details)
        self.setDefaultButton(self.defer_button)
        self.setIcon(self.Critical)
        self.defer_timer = QtCore.QTimer()
        self.defer_timer.timeout.connect(self.recenter)
        self.defer_timer.start( 1000 * self.timeout_seconds )

    def recenter(self):
        self.adjustSize()
        self.move(QtGui.QApplication.desktop().screen().rect().center() - self.rect().center())

    def handle_preempt(self):
        try:
            rospy.loginfo("Displaying ui_responder popup")
            self.exec_()
            while self.clickedButton() == self.defer_button:
                print "defer!"
                self.defer_timer.start( 1000 * self.timeout_seconds )
                self.move(0,0)
                self.exec_()

        except:
            pass

        rospy.loginfo("Exiting ui_responder")
        #TODO: Quit the application, and shutdown ROS
        rospy.signal_shutdown('quit')
        QtGui.qApp.quit()
        sys.exit()

    def req_cb(self, msg):
        self.preempted.emit()

def main():
    rospy.init_node('ui_responder', None, False, 2, False, False, True)
    app = QtGui.QApplication(rospy.myargv(sys.argv))
    ui_r = UIResponder()

    # app.exec_() returns when UIResponder widget closes.  Since we
    # want our defer_timer to keep ticking, keep calling exec_().
    # Program ends when sys.exit() is called explicitly.
    while True:
        app.exec_()

if __name__ == '__main__':
    main()
